@@ -572,7 +572,7 @@ impl Archetype {
         dst.reserve(1);
 
         debug_assert_ne!(dst.entities.len(), dst.entities.capacity());
-        self.relocate_components(src_entity_idx, dst, dst_entity_idx, |_, _| unsafe {
+        self.relocate_components(src_entity_idx, dst, dst_entity_idx, |_, _| {
             unreachable_unchecked()
         });
 
@@ -624,7 +624,7 @@ impl Archetype {
         dst.reserve(1);
 
         debug_assert_ne!(dst.entities.len(), dst.entities.capacity());
-        self.relocate_components(src_entity_idx, dst, dst_entity_idx, |_, _| unsafe {
+        self.relocate_components(src_entity_idx, dst, dst_entity_idx, |_, _| {
             unreachable_unchecked()
         });
 
@@ -670,7 +670,7 @@ impl Archetype {
         dst.reserve(1);
 
         debug_assert_ne!(dst.entities.len(), dst.entities.capacity());
-        self.relocate_components(src_entity_idx, dst, dst_entity_idx, |info, ptr| unsafe {
+        self.relocate_components(src_entity_idx, dst, dst_entity_idx, |info, ptr| {
             if info.id != TypeId::of::<T>() {
                 unreachable_unchecked()
             }
@@ -774,10 +774,8 @@ impl Archetype {
             let chunk_version = &mut *component.chunk_versions.as_ptr().add(chunk_idx);
             let entity_version = &mut *component.entity_versions.as_ptr().add(entity_idx);
 
-            unsafe {
-                debug_assert!(*component.version.get() <= epoch);
-                *component.version.get() = epoch;
-            }
+            debug_assert!(*component.version.get() <= epoch);
+            *component.version.get() = epoch;
 
             debug_assert!(*chunk_version <= epoch);
             *chunk_version = epoch;
@@ -785,14 +783,11 @@ impl Archetype {
             debug_assert!(*entity_version <= epoch);
             *entity_version = epoch;
 
-            unsafe {
-                let dst = component.ptr.as_ptr().add(entity_idx * size);
-
-                if occupied(id) {
-                    (component.set_one)(src.as_ptr(), dst);
-                } else {
-                    ptr::copy_nonoverlapping(src.as_ptr(), dst, size);
-                }
+            let dst = component.ptr.as_ptr().add(entity_idx * size);
+            if occupied(id) {
+                (component.set_one)(src.as_ptr(), dst);
+            } else {
+                ptr::copy_nonoverlapping(src.as_ptr(), dst, size);
             }
         });
     }
@@ -844,6 +839,7 @@ impl Archetype {
             let src_component = &self.components[src_type_idx];
             let size = src_component.layout.size();
             let type_id = src_component.id;
+            let src_ptr = src_component.ptr.as_ptr().add(src_entity_idx * size);
 
             if let Some(dst_type_idx) = dst.set.get(type_id) {
                 let dst_component = &dst.components[dst_type_idx];
@@ -867,36 +863,35 @@ impl Archetype {
                 debug_assert!(*dst_entity_version <= epoch);
                 *dst_entity_version = epoch;
 
-                let src_ptr = src_component.ptr.as_ptr().add(src_entity_idx * size);
                 let dst_ptr = dst_component.ptr.as_ptr().add(dst_entity_idx * size);
 
                 ptr::copy_nonoverlapping(src_ptr, dst_ptr, size);
-
-                if src_entity_idx != last_entity_idx {
-                    let src_chunk_idx = chunk_idx(src_entity_idx);
-
-                    let last_epoch = *src_component.entity_versions.as_ptr().add(last_entity_idx);
-
-                    let src_chunk_version =
-                        &mut *src_component.chunk_versions.as_ptr().add(src_chunk_idx);
-
-                    let src_entity_version =
-                        &mut *src_component.entity_versions.as_ptr().add(src_entity_idx);
-
-                    if *src_chunk_version < last_epoch {
-                        *src_chunk_version = last_epoch;
-                    }
-
-                    debug_assert!(*src_entity_version <= last_epoch);
-                    *src_entity_version = last_epoch;
-
-                    let last_ptr = src_component.ptr.as_ptr().add(last_entity_idx * size);
-                    ptr::copy_nonoverlapping(last_ptr, src_ptr, size);
-                }
             } else {
                 let src_ptr = src_component.ptr.as_ptr().add(src_entity_idx * size);
                 missing(src_component, src_ptr);
             }
+
+            if src_entity_idx != last_entity_idx {
+                let src_chunk_idx = chunk_idx(src_entity_idx);
+
+                let last_epoch = *src_component.entity_versions.as_ptr().add(last_entity_idx);
+
+                let src_chunk_version =
+                    &mut *src_component.chunk_versions.as_ptr().add(src_chunk_idx);
+
+                let src_entity_version =
+                    &mut *src_component.entity_versions.as_ptr().add(src_entity_idx);
+
+                if *src_chunk_version < last_epoch {
+                    *src_chunk_version = last_epoch;
+                }
+
+                debug_assert!(*src_entity_version <= last_epoch);
+                *src_entity_version = last_epoch;
+
+                let last_ptr = src_component.ptr.as_ptr().add(last_entity_idx * size);
+                ptr::copy_nonoverlapping(last_ptr, src_ptr, size);
+            }
         }
     }
 }
