@@ -16,7 +16,7 @@ use hashbrown::{
 };
 
 use crate::{
-    archetype::{Archetype, CHUNK_LEN_USIZE},
+    archetype::{chunk_idx, Archetype, CHUNK_LEN_USIZE},
     bundle::{Bundle, DynamicBundle},
     component::{Component, ComponentInfo},
     entity::{Entities, EntityId},
@@ -533,7 +533,7 @@ impl World {
 
         let (src, dst) = match archetype < insert_info.dst {
             true => (&mut before[archetype as usize], &mut after[0]),
-            false => (&mut after[0], &mut before[archetype as usize]),
+            false => (&mut after[0], &mut before[insert_info.dst as usize]),
         };
 
         let (dst_idx, opt_src_id) = unsafe { src.insert_bundle(dst, idx, bundle, self.epoch) };
@@ -591,7 +591,7 @@ impl World {
 
         let (src, dst) = match archetype < remove_info.dst {
             true => (&mut before[archetype as usize], &mut after[0]),
-            false => (&mut after[0], &mut before[archetype as usize]),
+            false => (&mut after[0], &mut before[remove_info.dst as usize]),
         };
 
         let (dst_idx, opt_src_id) = unsafe { src.drop_bundle(dst, idx) };
@@ -708,6 +708,9 @@ impl World {
         let (archetype, idx) = self.entities.get(entity).unwrap();
         let archetype = &self.archetypes[archetype as usize];
         let mut fetch = unsafe { Q::fetch(archetype, 0, self.epoch) }.expect("Query is prooven");
+        unsafe {
+            fetch.visit_chunk(chunk_idx(idx as usize));
+        }
         let item = unsafe { fetch.get_item(idx as usize) };
         item
     }
@@ -723,6 +726,8 @@ impl World {
     where
         Q: Query + ImmutableQuery + NonTrackingQuery,
     {
+        debug_assert!(Q::is_valid(), "Immutable queries are always valid");
+
         assert!(
             !Q::mutates(),
             "Invalid impl of `ImmutableQuery` for `{}`",
@@ -757,6 +762,8 @@ impl World {
     where
         Q: Query + NonTrackingQuery,
     {
+        assert!(Q::is_valid(), "Invalid query specified");
+
         assert!(
             !Q::tracks(),
             "Invalid impl of `NonTrackingQuery` for `{}`",
@@ -772,6 +779,9 @@ impl World {
         match unsafe { Q::fetch(archetype, 0, self.epoch) } {
             None => Err(EntityError::MissingComponents),
             Some(mut fetch) => {
+                unsafe {
+                    fetch.visit_chunk(chunk_idx(idx as usize));
+                }
                 let item = unsafe { fetch.get_item(idx as usize) };
                 Ok(item)
             }
@@ -877,6 +887,8 @@ impl World {
     where
         Q: Query + ImmutableQuery,
     {
+        debug_assert!(Q::is_valid(), "Immutable queries are always valid");
+
         QueryRef {
             epoch: self.epoch,
             archetypes: &self.archetypes,
@@ -893,6 +905,8 @@ impl World {
     where
         Q: Query,
     {
+        assert!(Q::is_valid(), "Invalid query specified");
+
         QueryMut {
             epoch: &mut self.epoch,
             archetypes: &self.archetypes,
@@ -912,6 +926,8 @@ impl World {
     where
         Q: Query,
     {
+        assert!(Q::is_valid(), "Invalid query specified");
+
         let meta = EntityMeta {
             entities: &mut self.entities,
             archetypes: &self.archetypes,
@@ -934,6 +950,7 @@ impl World {
         Q: Query + NonTrackingQuery + ImmutableQuery,
         F: FnMut(QueryItem<'_, Q>),
     {
+        debug_assert!(Q::is_valid(), "Immutable queries are always valid");
         debug_assert!(!Q::mutates());
 
         for archetype in &self.archetypes {
@@ -955,6 +972,7 @@ impl World {
         Q: Query + ImmutableQuery,
         F: FnMut(QueryItem<'_, Q>),
     {
+        debug_assert!(Q::is_valid(), "Immutable queries are always valid");
         debug_assert!(!Q::mutates());
 
         let tracks_epoch = tracks.epoch;
@@ -1004,6 +1022,8 @@ impl World {
         Q: Query + NonTrackingQuery,
         F: FnMut(QueryItem<'_, Q>),
     {
+        assert!(Q::is_valid(), "Invalid query specified");
+
         if Q::mutates() {
             self.epoch += 1
         };
@@ -1054,6 +1074,8 @@ impl World {
         Q: Query,
         F: FnMut(QueryItem<'_, Q>),
     {
+        assert!(Q::is_valid(), "Invalid query specified");
+
         if Q::mutates() {
             self.epoch += 1
         };
@@ -1642,7 +1664,7 @@ where
 
 /// Error returned in case specified [`EntityId`]
 /// does not reference any live entity in the [`World`].
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct NoSuchEntity;
 
 impl fmt::Display for NoSuchEntity {
@@ -1656,7 +1678,7 @@ impl std::error::Error for NoSuchEntity {}
 
 /// Error returned in case specified entity does not contain
 /// component of required type.
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct MissingComponents;
 
 impl fmt::Display for MissingComponents {
@@ -1670,7 +1692,7 @@ impl std::error::Error for MissingComponents {}
 
 /// Error returned if either entity reference is invalid
 /// or component of required type is not found for an entity.
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub enum EntityError {
     /// Error returned in case specified [`EntityId`]
     /// does not reference any live entity in the [`World`].
@@ -1712,9 +1734,21 @@ impl From<MissingComponents> for EntityError {
     }
 }
 
+impl PartialEq<NoSuchEntity> for EntityError {
+    fn eq(&self, _: &NoSuchEntity) -> bool {
+        matches!(self, EntityError::NoSuchEntity)
+    }
+}
+
+impl PartialEq<MissingComponents> for EntityError {
+    fn eq(&self, _: &MissingComponents) -> bool {
+        matches!(self, EntityError::MissingComponents)
+    }
+}
+
 /// Error that may occur when function expects `World` to own an entity with specific id.
 #[cfg(feature = "rc")]
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub enum OwnershipError {
     /// Error returned in case specified [`EntityId`]
     /// does not reference any live entity in the [`World`].
@@ -1732,6 +1766,13 @@ impl From<NoSuchEntity> for OwnershipError {
     }
 }
 
+#[cfg(feature = "rc")]
+impl PartialEq<NoSuchEntity> for OwnershipError {
+    fn eq(&self, _: &NoSuchEntity) -> bool {
+        matches!(self, OwnershipError::NoSuchEntity)
+    }
+}
+
 #[cfg(feature = "rc")]
 impl fmt::Display for OwnershipError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
